/***************************************************************************
 *  CISILIA PROJECT: cisilia-data.h
 *
 *  Tue Jan  7 23:13:39 2003
 *  Copyright  2002-2003  Cisilia Team <cisilia@cisiar.org>
 *
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
#include <sys/time.h>

/*---- V A R I A B L E S   D E   C O M P I L A C I O N -----------*/

#define LONG_PASSWD	            14  /* !< Longitud total de una clave */
#define BARRA_0                 '\0'    /* !< Caracter nulo en los strings */
#define POS_BARRA_0             0       /* !< Posicion del '\0' en los alfabetos */
#define ALFABETO_NUMERICO       1       /* !< Incluir los caracteres numericos al alfabeto por defecto */
#define CANT_PROCESOS_DEFAULT   1       /* !< Cantidad de procesos por defecto */
#define TIEMPO_ESTADISTICAS     100000  /* !< Intervalo entre envio de estadisticas [useg] */
#define LISTEN_QUEUE            10      /* !< Segundo parametro de la funcion listen() */
#define LOCALHOST               "127.0.0.1"     /* !< IP del proceso padre */
#define PUERTO                  6666    /* !< Puerto TCP/UDP del proceso padre */
#define BUSY_WAIT_SEC           1       /* !< segundos para select() */
#define BUSY_WAIT_USEC          0       /* !< microsegundos para select() */
#define CONEXION_PERMANENTE     1       /* !< Tipo de comunicacion entre hijos y padre */
#define SUBPROC_GEN_PAL_NULA    1       /* !< Generar la palabra nula en un subproceso particular */
#define CRACKEAR(x,y)           crackear_passwords_5(x,y)       /* !< Funcion de generacion de palabras */
#define BUSCAR(x,y)             buscar_LMhash_matriz_bin(x,y)   /* !< Funcion de busqueda de los semiLMhashes de las palabras generadas */

/*--------------------------------------------------*/

typedef unsigned char uchar;
typedef unsigned short int16;
typedef unsigned short uint16;

// ! Parametros de la Linea de Comandos
typedef struct {
    unsigned int procesos;      /**< Cantidad de procesos hijos */
    char verbose;               /**< Imprimir informacion de depuracion */
//    char *arch_conf;          /**< Archivo de Configuracion */
    char *arch_SAM;             /**< Archivo de claves */
    char *arch_alfabeto;        /**< Archivo de alfabeto */
    char *arch_log;             /**< Archivo de vitacora de claves rotas */
    char *arch_dict;            /**< Archivo de diccionario */
    int puerto;                 /**< Puerto de escucha del proceso padre */
    char *frontend_host;        /**< Host del front-end */
    int frontend_puerto;        /**< Puerto de escucha del front-end */
} param;

// ! Posiciones en el alfabeto para calcular un rango de passwords
//typedef struct {
//    int desde[LONG_PASSWD / 2];       /* !< Limite inferior del rango */
//    int hasta[LONG_PASSWD / 2];       /* !< Limite superior del rango */
//} rango_passwd;

// ! Estructura de Datos de Usuarios
typedef struct {
    uchar nombre[20];           /* !< Nombre del usuario */
    uchar passwd[LONG_PASSWD + 1];      /* !< Clave del usuario */
    uchar NThash[16];           /* !< Hash MD4 de la clave del usuario */
    uchar ok;                   /* !< Estado del cracking de la clave - 0:nada IZQ DER PASSWD_OK */
} datoUser;

// ! Estructura de Datos de LanManager Hashes 
typedef struct {
    uchar LMsemihash[8];        /* !< Mitad del LanManager Hash del usuario */
    int usuario;                /* !< Posicion del usuario en la estructura de usuarios */
    char mitad;                 /* !< izq/der */
} LMhash;

// ! Estructura de Datos de los Mensajes
typedef struct {
    uchar semipasswd[LONG_PASSWD / 2 + 1];      /* !< Mitad del LanManager Hash del usuario */
    int usuario;                /* !< Posicion del usuario en la estructura de usuarios */
    char mitad;                 /* !< izq/der */
    int pid;                    /* !< PID del subproceso emisor del mensaje */
} crack;

// ! Estructura de Datos del indice de LMhashes
typedef struct {
    int pos;                    /* !< Posicion del primer LMhash que comienza con ese caracter en LMHashes */
    unsigned int cant;          /* !< Cantidad de lmhashes que comienzan con ese caracter */
} indice;

// ! Estructura de Datos en el padre de cada hijo
//typedef struct subproc
//{
//  int socket;
//  int pid;
//}
//subproc;

typedef struct {
    int pid;
    int lineas;
    struct timeval ti;          /* !< Hora inicial del proceso */
    struct timeval t;           /* !< Hora actual */
    unsigned long rx;           /* !< Bytes recibidos en mensajes */
    unsigned long long total_palabras;
    unsigned long long palabras;
//    unsigned long long invalidas;
    unsigned long long semicracks;
    unsigned long long cracks;
} estadisticas;

typedef struct {
    int pid;
    int id_hijo;
    unsigned long long palabras;
} datos_hijo;

typedef struct {
    unsigned long long cracks;
    struct timeval ti;          /* !< Hora inicial del proceso */
    struct timeval t;           /* !< Hora actual */
    uchar nombre[20];           /* !< Nombre del usuario */
    uchar LMpasswd[LONG_PASSWD + 1];    /* !< Clave LanManager del usuario */
    uchar NTpasswd[LONG_PASSWD + 1];    /* !< Clave NT/MD4 del usuario */
} datoCrack;

typedef union {
    estadisticas data;
    datoCrack crack;
} msgPadre;

typedef union {
    int puerto;
    datos_hijo data;
} msgHijo;
