/***************************************************************************
 *  CISILIA PROJECT: cisilia.h
 *
 *  Tue Jan  7 23:13:39 2003
 *  Copyright  2002-2003  Cisilia Team <cisilia@cisiar.org>
 *
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 
#include "cisilia-data.h"
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
/* fork() & getopt() & getrusage() & alarm() */
#include <unistd.h>
/* getopt_long() */
#include <getopt.h>
#define _GNU_SOURCE
/* fork() */
#include <sys/types.h>
#include <sys/wait.h>
/* getrusage() & gettimeofday() */
#include <sys/time.h>
#include <sys/resource.h>
/* times() */
#include <sys/times.h>
#include <time.h>
/* comunicaciones */
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <arpa/inet.h>
/* soporte para cantidad variable de argumentos a funciones */
#include <stdarg.h>
/* MD4 */
#include "byteorder.h"		/* ! Cortesia de SMB */
#include "md4.h"		/* ! RFC1186 */

/*-------------- C O N S T A N T E S -----------------------------*/

#define OK			0	/* !< Valor devuelto si no hay errores */
#define IZQ			1	/* !< Mitad izquierda del LMHash */
#define DER			2	/* !< Mitad derecha del LMHash */
#define PADRE       0   /* !< Jerarquia del proceso padre */
#define HIJO        0   /* !< Jerarquia de los procesos hijos */
#define ENCONTRADO	 	1	/* !< Flag de clave encontrada */
#define PASSWD_OK		IZQ+DER	/* !< Clave totalmente descifrada */
#define HASH_INEXISTENTE	-1	/* !< Hash NT del archivo Invalido */
#define LONG_ALF_ASCII 		256	/* !< Obvio... */
#define ERROR_FORK		0	/* !< Codigo de error */
#define ERROR_MALLOC		1	/* !< Codigo de error */
#define ERROR_ARCH_PASSWD	2	/* !< Codigo de error */
#define ERROR_ARCH_ALFABETO	3	/* !< Codigo de error */
#define ERROR_ARCH_DIC		15	/* !< Codigo de error */
#define ERROR_REALLOC		4	/* !< Codigo de error */
#define ERROR_SOCKET		5	/* !< Codigo de error */
#define ERROR_SOCKET_SEND	6	/* !< Codigo de error */
#define ERROR_SOCKET_RECV	7	/* !< Codigo de error */
#define ERROR_SOCKET_BIND	8	/* !< Codigo de error */
#define ERROR_SOCKET_LISTEN	9	/* !< Codigo de error */
#define ERROR_SOCKET_ACCEPT	10	/* !< Codigo de error */
#define ERROR_SOCKET_HOSTNAME	11	/* !< Codigo de error */
#define ERROR_SOCKET_SELECT	12	/* !< Codigo de error */
#define ERROR_SOCKET_CONN	13	/* !< Codigo de error */
#define ERROR_SOCKET_SIGPIPE	14	/* !< Codigo de error */
#define CONSTANTE_MAGICA	"\x4B\x47\x53\x21\x40\x23\x24\x25"
				/* !<  Cadena constante de entrada al Algoritmo DES */

/*----------------------------------------------------*/

void cicloPrincipal();
int cargar_alfabeto(uchar *);
int archivo_a_memoria(uchar *);
//int imprimir_datos_users_plano();
int ordenar_LMhashes(int);
int generar_indice_LMhashes(int);
int comparar_LMhashes(const void *, const void *);
//double calcular_rango_total(void);
double total_palabras_validas(void);
//double dividir_rango_total(int, int *);
//int cambiar_base(double, int, int *, int);
//int sumar_base_n(const int *, const int *, const int, const int, int *);
//static inline int sumar_uno_base_n(int *, const int, const int);
//int asignar_rango(int *, int *, const int *);
//int init_rango(int *, int);
//int crackear_passwords_4(const int *, const int *);
int crackear_passwords_5(const int *, const int *);
int crackear_passwords_dic(uchar *);
//int crackear_passwords_palabra_nula(void);
int buscar_LMhash_bin(const uchar *, const uchar *, int, int);
//int buscar_LMhash_bin_total(const uchar *, const uchar *);
//int buscar_LMhash_indice(const uchar *, const uchar *);
//int buscar_LMhash_indice_bin(const uchar *, const uchar *);
int buscar_LMhash_indice_bin_extra(const uchar *, const uchar *);
int buscar_LMhash_matriz_bin(const uchar *, const uchar *);
int recibirDatosUsers(const uchar *, const int, const char);
int enviarDatosUsers(const uchar *, const int, const int);
int chequear_passwd_ok(const int);
static int calcularPwdMD_2(uchar *, int, int, int);
int calcularNThash(uchar *, uchar *);
void error(int);
extern void E1(uchar *, uchar *, uchar *);
int leer_opciones(const int, char **);
void ayuda(const char *);
void imprimir_estadisticas(void);
void version(void);
void cerrar_sockets(void);
void salir();
void capturar_senial(int);
void inicioSubproc(void);
void inicioPrincipal(void);
void inicioPadre(void);
void matarSubProcesos(void);
int crearSocketEscuchaTCP(int);
int crearSocketEscuchaUDP(int);
int conectarTCP(const char *, const int);
int conectarUDP(const char *, const int);
static int escuchar();
void enviar_datos_hijo();
void enviar_estadisticas();
int mensaje(const char *, ...);
int advertencia(const char *);
int logear(const char *, ...);

/*--------------------------------------------------*/
